<?php
/**
 * Elementor oEmbed Widget.
 *
 * Elementor widget that inserts an embbedable content into the page, from any given URL.
 *
 * @since 1.0.0
 */
class ACF_Elementor_Form_Widget extends \Elementor\Widget_Base {
	
	/**
	 * Get widget name.
	 *
	 * Retrieve acf ele form widget name.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'acf_ele_form';
	}

	/**
	 * Get widget title.
	 *
	 * Retrieve acf ele form widget title.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return __( 'ACF Frontend Form', 'acf-ele-form' );
	}

	/**
	 * Get widget icon.
	 *
	 * Retrieve acf ele form widget icon.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'fa fa-wpforms';
	}

	/**
	 * Get widget categories.
	 *
	 * Retrieve the list of categories the acf ele form widget belongs to.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'general' ];
	}

	/**
	 * Register acf ele form widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function _register_controls() {
		
		//get all field group choices
		$field_group_choices = $field_choices = array();
		$front_groups = acf_get_field_groups();
		// loop through array and add to field 'choices'
		if( is_array( $front_groups ) ) {        
			foreach( $front_groups as $front_group ) {
				
				//get options for "field groups select" field
				$field_group_choices[ $front_group['key'] ] = $front_group['title']; 
				
				//get options for "fields select" field
				$front_fields = get_posts(array(
				'posts_per_page'   => -1,
				'post_type'        => 'acf-field',
				'orderby'          => 'menu_order',
				'order'            => 'ASC',
				'suppress_filters' => true, // DO NOT allow WPML to modify the query
				'post_parent'      => $front_group['ID'],
				'post_status'      => 'any',
				'update_post_meta_cache' => false
			  ));
				// loop through array and add to field 'choices'
				if( is_array( $front_fields ) ) { 
					foreach( $front_fields as $front_field ) {
						$field_choices[ $front_field->post_name] = $front_field->post_title . ' (' . $front_group['title'] . ')';   
					}
				} 
			}
		}
		
		$who_can_see = array( 'logged_in' => __( 'Only Logged In Users', 'acf-ele-form' ), 'logged_out' => __( 'Only Logged Out', 'acf-ele-form'), 'all' => __( 'All Users', 'acf-ele-form' ) );
			
		//get all user roles choices
		$user_roles = array();        
		// get the textarea value from options page without any formatting
		global $wp_roles;
		$roles = $wp_roles->get_names();
		// loop through array and add to field 'choices'
		if( is_array( $roles ) ) {        
			foreach( $roles as $role ) {
				$user_roles[ str_replace( ' ', '_', strtolower( $role ) ) ] = $role;   
			}
		}
		
		//get all users choices
		$user_choices = array();
		$users = get_users();
		foreach($users as $user){
			$user_choices[$user->ID] = $user->display_name;
		}
		
		//get all post types choices
		$post_types_choices = array();        
		// get the textarea value from options page without any formatting
		$args = array(
			'public'   => true,
		);
		$output = 'names'; // names or objects, note names is the default
		$operator = 'and'; // 'and' or 'or'
		$post_types = get_post_types( $args, $output, $operator ); 
		// loop through array and add to field 'choices'
		if( is_array( $post_types ) ) {        
			foreach( $post_types as $post_type ) {
				$post_types_choices[ $post_type ] = str_replace( '_', ' ', ucfirst( $post_type ) ); 
			}
		}
		
		//get all posts choices
		$posts_choices = array();
		$posts = get_posts( array( 'posts_per_page' => '-1', 'post_type' => 'any' ) ); 
		// loop through array and add to field 'choices'
		if( is_array( $posts ) ) {        
			foreach( $posts as $p ) {
				$posts_choices[ $p->ID ] = $p->post_title;   
			}
		}
		

		$this->start_controls_section(
			'fields_section',
			[
				'label' => __( 'Fields', 'acf-ele-form' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'form_title',
			[
				'label' => __( 'Form Title', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => __( 'Edit Post', 'acf-ele-form' ),
				'placeholder' => __( 'Edit Post', 'acf-ele-form' ),
			]
		);
		
		$this->add_control(
			'show_title',
			[
				'label' => __( 'Show Title Field', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Show', 'acf-ele-form' ),
				'label_off' => __( 'Hide','acf-ele-form' ),
				'return_value' => 'true',
				'default' => 'true',
			]
		);
		
		$this->add_control(
			'show_content',
			[
				'label' => __( 'Show Content Field', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Show', 'acf-ele-form' ),
				'label_off' => __( 'Hide','acf-ele-form' ),
				'return_value' => 'true',
			]
		);
		
		$this->add_control(
			'form_fields',
			[
				'label' => __( 'Form Fields', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'label_block' => true,
				'default' => 'current',
				'options' => [
					'current'  => __( 'Current Post\'s Field Groups', 'acf-ele-form' ),
					'field_groups' => __( 'Select Field Groups', 'acf-ele-form' ),
					'fields' => __( 'Select Fields', 'acf-ele-form' ),
				],
			]
		);
		
		$this->add_control(
			'field_groups_select',
			[
				'label' => __( 'Field Groups', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'multiple' => true,
				'options' => $field_group_choices,
				'condition' => [
					'form_fields' => 'field_groups',
				],
			]
		);
		$this->add_control(
			'fields_select',
			[
				'label' => __( 'Fields', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'label_block' => true,
				'multiple' => true,
				'options' => $field_choices,
				'condition' => [
					'form_fields' => 'fields',
				],
			]
		);
		
		$this->add_control(
			'submit_button_text',
			[
				'label' => __( 'Submit Button Text', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => __( 'Update', 'acf-ele-form' ),
				'placeholder' => __( 'Update', 'acf-ele-form' ),
			]
		);
		
		$this->end_controls_section();

		
		$this->start_controls_section(
			'display_section',
			[
				'label' => __( 'Display', 'acf-ele-form' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		
		$this->add_control(
			'show_in_modal',
			[
				'label' => __( 'Show in Modal', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Yes', 'acf-ele-form' ),
				'label_off' => __( 'No','acf-ele-form' ),
				'return_value' => 'true',
			]
		);
		$this->add_control(
			'modal_button_text',
			[
				'label' => __( 'Modal Button Text', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => __( 'Edit', 'acf-ele-form' ),
				'placeholder' => __( 'Edit', 'acf-ele-form' ),
				'condition' => [
					'show_in_modal' => 'true',
				],
			]
		);
		
		$this->end_controls_section();
		
		$this->start_controls_section(
			'permissions_section',
			[
				'label' => __( 'Permissions', 'acf-ele-form' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		
												
		$this->add_control(
			'who_can_see',
			[
				'label' => __( 'Who Can See This...', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'default' => 'logged_in',
				'options' => $who_can_see,
			]
		);
			
		$this->add_control(
			'by_role',
			[
				'label' => __( 'Select By Role', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'multiple' => true,
				'default' => 'administrator',
				'options' => $user_roles,
				'condition' => [
					'who_can_see' => 'logged_in',
				],
			]
		);
		$this->add_control(
			'user_id',
			[
				'label' => __( 'Select By User', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'multiple' => true,
				'options' => $user_choices,
				'default' => get_current_user_id(),
				'condition' => [
					'who_can_see' => 'logged_in',
				],
			]
		);
		$this->add_control(
			'dynamic',
			[
				'label' => __( 'Dynamic Selection', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'description' => 'Use a dynamic meta key that returns a user ID to filter the form for that user dynamically. You may also use [author] for the post\'s author',
				'default' => '[author]',
				'placeholder' => '[author]',
				'condition' => [
					'who_can_see' => 'logged_in',
				],
			]
		);
		
		$this->end_controls_section();

		$this->start_controls_section(
			'actions_section',
			[
				'label' => __( 'Form Actions', 'acf-ele-form' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		
		$this->add_control(
			'main_action',
			[
				'label' => __( 'Main Action', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'edit_post',
				'options' => [
					'edit_post'  => __( 'Edit Post', 'acf-ele-form' ),
					'edit_user' => __( 'Edit User', 'acf-ele-form' ),
					'new_post' => __( 'New Post', 'acf-ele-form' ),
				],
			]
		);
		
		$this->add_control(
			'post_to_edit',
			[
				'label' => __( 'Post To Edit', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'current_post',
				'options' => [
					'current_post'  => __( 'Current Post', 'acf-ele-form' ),
					'select_post' => __( 'Select Post', 'acf-ele-form' ),
				],
				'condition' => [
					'main_action' => 'edit_post',
				],
			]
		);
		
		$this->add_control(
			'post_select',
			[
				'label' => __( 'Post', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'options' => $posts_choices,
				'conditions' => [
					'relation' => 'and',
					'terms' => [
						[
							'name' => 'post_to_edit',
							'operator' => '==',
							'value' => 'select_post',
						],
						[
							'name' => 'main_action',
							'operator' => '==',
							'value' => 'edit_post',
						],
					],
				],
			]
		);
		
		$this->add_control(
			'user_to_edit',
			[
				'label' => __( 'User To Edit', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'current_user',
				'options' => [
					'current_user'  => __( 'Current User', 'acf-ele-form' ),
					'select_user' => __( 'Select User', 'acf-ele-form' ),
				],
				'condition' => [
					'main_action' => 'edit_user',
				],
			]
		);
		
		$this->add_control(
			'user_select',
			[
				'label' => __( 'User', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'options' => $user_choices,
				'default' => get_current_user_id(),
				'conditions' => [
					'relation' => 'and',
					'terms' => [
						[
							'name' => 'user_to_edit',
							'operator' => '==',
							'value' => 'select_user',
						],
						[
							'name' => 'main_action',
							'operator' => '==',
							'value' => 'edit_user',
						],
					],
				],
			]
		);
		
		
		$this->add_control(
			'new_post_type',
			[
				'label' => __( 'New Post Type', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'default' => 'post',
				'options' => $post_types_choices,
				'condition' => [
					'main_action' => 'new_post',
				],
			]
		);
		
		$this->add_control(
			'new_post_status',
			[
				'label' => __( 'New Post Status', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'default' => 'draft',
				'options' => [
					'draft' => __( 'Draft', 'acf-ele-form' ),
					'private' => __( 'Private', 'acf-ele-form' ),
					'pending' => __( 'Pending Review', 'acf-ele-form' ),
					'publish'  => __( 'Published', 'acf-ele-form' ),
				],
				'condition' => [
					'main_action' => 'new_post',
				],
			]
		);
		
		$this->add_control(
			'redirect',
			[
				'label' => __( 'Redirect After Submit', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'current',
				'options' => [
					'current'  => __( 'Reload Current Page/Post', 'acf-ele-form' ),
					'post_url' => __( 'Form Post Url', 'acf-ele-form' ),
					'custom_url' => __( 'Custom Url', 'acf-ele-form' ),
				],
			]
		);
		
		$this->add_control(
			'custom_url',
			[
				'label' => __( 'Custom Url', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::URL,
				'placeholder' => __( 'Enter Url Here', 'acf-ele-form' ),
				'condition' => [
					'redirect' => 'custom_url',
				],
			]
		);
		$this->add_control(
			'update_message',
			[
				'label' => __( 'Update Message', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => __( 'Your post has been updated', 'acf-ele-form' ),
				'placeholder' => __( 'Your post has been updated', 'acf-ele-form' ),
			]
		);
		$this->add_control(
			'show_delete_button',
			[
				'label' => __( 'Delete Post Option', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Yes', 'acf-ele-form' ),
				'label_off' => __( 'No','acf-ele-form' ),
				'return_value' => 'true',
				'condition' => [
					'main_action' => 'edit_post',
				],
			]
		);
		$this->add_control(
			'delete_button_text',
			[
				'label' => __( 'Delete Button Text', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => __( 'Delete', 'acf-ele-form' ),
				'placeholder' => __( 'Delete', 'acf-ele-form' ),
				'condition' => [
					'show_delete_button' => 'true',
				],
			]
		);
		$this->add_control(
			'confirm_delete_message',
			[
				'label' => __( 'Confirm Delete Message', 'acf-ele-form' ),
				'type' => \Elementor\Controls_Manager::TEXTAREA,
				'rows' => 3,
				'default' => __( 'The post will be deleted. Are you sure?', 'acf-ele-form' ),
				'placeholder' => __( 'The post will be deleted. Are you sure?', 'acf-ele-form' ),
				'condition' => [
					'show_delete_button' => 'true',
				],
			]
		);
		
		
		$this->end_controls_section();


	}

	/**
	 * Render acf ele form widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function render() {
		$wg_id = $this->get_id();
		$active_user = wp_get_current_user();
		$settings = $this->get_settings_for_display();
		$field_groups = $fields = $post_id = $new_post = false;
		$redirect_url =  '?updated=true';
		$submit_button = '<input type="submit" class="acf-button button button-primary button-large" value="%s" />';
		$update_message_html = '';
		
		if( true == $GET['updated'] ){
			$update_message_html = '<div id="message" class="updated published"><p>' . $settings['update_message'] . '</p></div>';
		}
		
		if( 'new_post' == $settings['main_action'] ){
			$post_id = 'new_post';
			$new_post = array( 'post_type' => $settings['new_post_type'], 'post_status'	=> $settings['new_post_status'] );
		}
		if( 'edit_post' == $settings['main_action'] ){
			if( $settings['post_to_edit'] == 'select_post' ){
				$post_id = $settings['post_select'];
			}else{
				$post_id = false;
			}
		}
		if( 'edit_user' == $settings['main_action'] ){
			if( $settings['user_to_edit'] == 'select_user' ){
				$post_id = 'user_' . $settings['user_select'];
			}else{
				$post_id = 'user_' . $active_user->ID;
			} 
		}	
		
		if( 'field_groups' == $settings['form_fields'] ){
			$field_groups = $settings['field_groups_select'];
		}
		if( 'fields' == $settings['form_fields'] ){
			$fields = $settings['fields_select'];
		}
		$visible = false;
		if( 'all' == $settings['who_can_see'] ){
			$visible = true;
		}elseif( 'logged_out' == $settings['who_can_see'] ){
			$visible = ( is_user_logged_in() ) ? false : true;
		}else{
			if( ! is_user_logged_in() ){
				$visible = false;
			}else{
				if( is_array( $settings['by_role'] ) ){
					if ( count( array_intersect( $settings['by_role'], (array) $active_user->roles ) ) == 0 ) {
						$by_role = false;
					}else{
						$by_role = true;
					}		
				}
				if( is_array( $settings['user_id'] ) ){
					if( in_array( $active_user->ID, $settings['user_id'] ) ){
						$user_id = true;
					}else{
						$user_id = false;
					}
				}
				if( $settings['dynamic'] ){
					if( '[author]' == $settings['dynamic'] ){
						$current_user = get_the_author_meta( 'ID' );
					}else{
						$current_user = get_post_meta( get_the_ID(), $settings['dynamic'] );	
					}
					if( $current_user == $active_user->ID ){
						$dynamic = true;
					}else{
						$dynamic = false;
					}
				}
				if( $by_role || $user_id || $dynamic ){
					$visible = true;
				}else{
					$visible = false;
				}
			}
		}
		if( 'post_url' == $settings['redirect'] ){
			$redirect_url = '%post_url%' . $redirect_url;
		}
		if( 'custom_url' == $settings['redirect'] ){
			$redirect_url = $settings['custom_url'] . $redirect_url;
		}
		
		if( $settings['show_delete_button'] ){ 
			$submit_button .= '<a class="acf-button button button-danger button-large" onclick="return confirm(\'' . $settings['confirm_delete_message'] . '\')" href="' . get_delete_post_link( $post->ID ) . '">' . $settings['delete_button_text'] . '</a>';
		}
		
		$form_args = array( 'post_id' => $post_id, 'new_post' => $new_post, 'post_title' => $settings['show_title'], 'post_content' => $settings['show_content'], 'field_groups' => $field_groups, 'fields' => $fields, 'submit_value' => $settings['submit_button_text'], 'html_submit_button' => $submit_button, 'return' => $redirect_url, 'updated_message' => $settings['update_message'], 'html_updated_message' => $update_message_html );
		
		if( $visible ){
			echo $update_message_html;
			if( $settings['show_in_modal'] !== 'true' ){
				if( $settings['form_title'] ){
					echo '<h2>' . $settings['form_title'] . '</h2>';
				}
				acf_form( $form_args );
			}else{
			?>
				<button class="edit-button" onClick="openModal('<?php echo $wg_id ?>')" ><?php echo $settings['modal_button_text'] ?></button>

				<!-- The Modal -->
				<div id="editModal_<?php echo $wg_id ?>" class="modal edit-modal">
					
				  <!-- Modal content -->
				  <div class="modal-content"> 
					<span onClick="closeModal('<?php echo $wg_id ?>')" class="close">&times;</span>
					<?php if( $settings['form_title'] ){
						echo '<h2>' . $settings['form_title'] . '</h2>';
					}  
					acf_form( $form_args ); ?>
				  </div>
				</div>
			<?php
			}
		}
	}
	

}